<?php
/*
 * Copyright (c) 2021. IOWEB TECHNOLOGIES
 */

namespace Ioweb\Base\Helper;


use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product\Type;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\InventoryConfiguration\Model\StockItemConfiguration;
use Magento\InventoryConfigurationApi\Api\GetStockItemConfigurationInterface;
use Magento\InventoryConfigurationApi\Exception\SkuIsNotAssignedToStockException;
use Magento\InventorySalesApi\Api\AreProductsSalableInterface;
use Magento\InventorySalesApi\Api\GetProductSalableQtyInterface;
use Magento\InventorySalesApi\Api\StockResolverInterface;
use Magento\InventorySalesApi\Model\GetStockItemDataInterface;
use Magento\InventorySalesApi\Model\StockByWebsiteIdResolverInterface;
use Magento\Store\Model\StoreManagerInterface;

class Msi extends AbstractHelper
{

    protected $areProductsSalable;
    protected $stockResolver;
    protected $stockByWebsiteIdResolver;
    protected $storeManager;

    protected $websiteId;
    protected $stockId;
    protected $getProductSalableQty;
    protected $getStockItemData;
    protected $getStockItemConfig;

    public function __construct(
        Context                            $context,
        AreProductsSalableInterface        $areProductsSalable,
        StockResolverInterface             $stockResolver,
        StockByWebsiteIdResolverInterface  $stockByWebsiteIdResolver,
        StoreManagerInterface              $storeManager,
        GetProductSalableQtyInterface      $getProductSalableQty,
        GetStockItemDataInterface          $getStockItemData,
        GetStockItemConfigurationInterface $getStockItemConfig

    )
    {
        $this->areProductsSalable = $areProductsSalable;
        $this->stockResolver = $stockResolver;
        $this->stockByWebsiteIdResolver = $stockByWebsiteIdResolver;
        $this->storeManager = $storeManager;
        $this->getProductSalableQty = $getProductSalableQty;
        $this->getStockItemData = $getStockItemData;
        $this->getStockItemConfig = $getStockItemConfig;
        parent::__construct($context);

        $this->websiteId = $this->storeManager->getWebsite()->getId();
        $this->stockId = $this->stockByWebsiteIdResolver->execute($this->websiteId)->getStockId();
    }


    /**
     * @param $product ProductInterface
     */
    public function isProductSalableInStoreView($product)
    {
        return $this->areProductsSalable->execute([$product->getSku()], $this->stockId);
    }


    /**
     * @param $product ProductInterface|Product
     * @param $stockId
     * @return boolean
     */
    public function isProductSalableInStock($product, $stockId)
    {
        if ($product->getTypeId() === Configurable::TYPE_CODE) {
            return $this->hasSaleableChildren($product, $stockId);
        }

        $stockData = $this->getStockItemData->execute($product->getSku(), $stockId);

        if (!empty($stockData)) {
            return ((bool)$stockData[GetStockItemDataInterface::IS_SALABLE] && $this->hasQty($product, $stockId));
        }
        return false;
    }

    /**
     * @var $product ProductInterface|Product
     */
    public function hasSaleableChildren($product, $stockId)
    {
        /** @var Configurable $configurable */
        $configurable = $product->getTypeInstance();
        $children = $configurable->getUsedProducts($product);
        foreach ($children as $child) {
            if ($this->isSalableSimpleProduct($child, $stockId)) {
                return true;
            }
        }
        return false;
    }

    public function isSalableSimpleProduct($product, $stockId)
    {
        $salableArray = $this->areProductsSalable->execute([$product->getSku()], $stockId);
        foreach ($salableArray as $salableResult) {
            if ((bool)$salableResult->isSalable()) {
                return true;
            }
        }
    }

    /**
     * @param $product ProductInterface|Product
     * @param $stockId int
     * @return bool
     */
    public function hasQty($product, $stockId)
    {

        switch ($product->getTypeId()) {
            case Configurable::TYPE_CODE:
                /** @var Configurable $instace */
                $instace = $product->getTypeInstance();
                $children = $instace->getUsedProducts($product);
                foreach ($children as $child) {
                    if (($this->getProductQtyInStock($child, $stockId) > 0) || $this->isAllowedBackorder($child, $stockId)) {
                        return true;
                    }                }
                break;
            case Type::TYPE_SIMPLE:
                if (($this->getProductQtyInStock($product, $stockId) > 0) || $this->isAllowedBackorder($product, $stockId)) {
                    return true;
                }

                break;
            default:
                return false;

        }
        return false;
    }

    /**
     * @param $product ProductInterface|Product
     * @param $stockId int
     * @return bool
     */
    public function hasPureQty($product, $stockId)
    {

        switch ($product->getTypeId()) {
            case Configurable::TYPE_CODE:
                /** @var Configurable $instace */
                $instace = $product->getTypeInstance();
                $children = $instace->getUsedProducts($product);
                foreach ($children as $child) {
                    if ($this->getProductQtyInStock($child, $stockId) > 0) {
                        return true;
                    }                }
                break;
            case Type::TYPE_SIMPLE:
                if ($this->getProductQtyInStock($product, $stockId) > 0) {
                    return true;
                }

                break;
            default:
                return false;

        }
        return false;
    }

    /**
     * @param $product ProductInterface
     * @param $stockId
     * @return float
     * @throws InputException
     * @throws LocalizedException
     */
    public function getProductQtyInStock($product, $stockId)
    {
        $stockData = $this->getStockItemData->execute($product->getSku(), $stockId);
        if (!empty($stockData)) {
            return $stockData[GetStockItemDataInterface::QUANTITY];
        }

        return 0;
    }

    /**
     * @param $product ProductInterface|Product
     * @param $stockId
     * @return bool
     * @throws LocalizedException
     * @throws \Magento\InventoryConfigurationApi\Exception\SkuIsNotAssignedToStockException
     */
    public function isAllowedBackorder($product, $stockId)
    {
        try {
            $stockItemConfig = $this->getStockItemConfig->execute($product->getSku(), $stockId);
        } catch (SkuIsNotAssignedToStockException $skuIsNotAssignedToStockException) {
            return false;
        }

        return $stockItemConfig->getBackorders() !== StockItemConfiguration::BACKORDERS_NO;
    }


}